﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  public abstract class FillMissingValueCommandBase : ColumnGroupCommandWithAffectedColumnsBase {
    protected Dictionary<int, List<int>> nullValues;
    protected ICollection<int> oldSortedColumnIndexes;
    protected IEnumerable<SortOrder> oldSortOrder;

    private FillMissingValueCommandBase()
      : base(null, string.Empty, null) {
      nullValues = new Dictionary<int, List<int>>();
    }

    protected FillMissingValueCommandBase(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
      nullValues = new Dictionary<int, List<int>>();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ColumnBase column;
      foreach (int col in nullValues.Keys) {
        column = ColumnGroup.GetColumn(col);
        for (int j = 0; j < this.nullValues[col].Count; j++)
          column.ChangeValue(this.nullValues[col][j], null);
      }
      ColumnGroup.SortOrdersForColumns = oldSortOrder;
      ColumnGroup.SortedColumnIndexes = oldSortedColumnIndexes;
      oldSortedColumnIndexes = null;
      oldSortOrder = null;
      nullValues.Clear();
      ColumnGroup.FireChanged();
      ColumnGroup = null;
    }
  }
}
